///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics;
using ILNumerics.Exceptions;
using ILNumerics.Storage;
using ILNumerics.Misc;

namespace ILNumerics {

    public partial class ILMath {

        /// <summary>
        /// Size of dimensions of A 
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>Double array with the length of each dimension of A</returns>
        /// <remarks>If A is null, an empty array will be returned. Otherwise the array returned will always be a row vector of length s. s &gt;= 2</remarks>
        public static ILRetArray<double> size(ILBaseArray A) {
            using (ILScope.Enter(A)) {
                if (object.Equals (A,null))
                    return empty<double>(ILSize.Empty00); 
                int numDim = A.Size.NumberOfDimensions;
                double [] retArr = new double[numDim]; 
                for (int i = 0; i < numDim; i++) {
                    retArr[i] = A.Size[i];     
                }
                return new ILRetArray<double>(retArr,1,numDim); 
            }
        }
        /// <summary>
        /// Length of one specific dimension of A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <param name="dim">Number of dimension to query the length for</param>
        /// <returns>Length of dimension 'dim'</returns>
        public static int size(ILBaseArray A, int dim) {
            using (ILScope.Enter(A)) {
                if (object.Equals(A,null) || dim < 0)
                    throw new ILArgumentException("size: A must not be null");
                return A.Size[dim];
            }
        }
        /// <summary>
        /// Create a size descriptor
        /// </summary>
        /// <param name="dimensions">Arbitrary number of dimension length</param>
        /// <returns>Size descriptor</returns>
        public static ILSize size(params int[] dimensions) {
            return new ILSize(dimensions); 
        }

        /// <summary>
        /// Longest dimension of A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>If A is null:0 - length of longest dimension of A</returns>
        /// <remarks>This is an alias/abreviation for A.Dimensions.Longest</remarks>
        public static int length(ILBaseArray A) {
            using (ILScope.Enter(A)) {
                if (object.Equals(A,null)) return 0;
                return A.Size.Longest;
            }
        }
        
        /// <summary>
        /// Number of dimensions of A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>If A is null: 0 - else number of dimensions of A</returns>
        /// <remarks>This is an alias/abreviation for A.Dimensions.NumberOfDimensions</remarks>
        /// 
        public static int ndims(ILBaseArray A) {
            using (ILScope.Enter(A)) {
                if (object.Equals(A,null)) return 0;
                return A.Size.NumberOfDimensions;  
            }
        }

        /// <summary>
        /// Number of elements of A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>Number of elements of A</returns>
        /// <remarks>This is an alias/abreviation for A.Dimensions.NumberOfElements</remarks>
        public static int numel(ILBaseArray A) {
            using (ILScope.Enter(A)) {
                if (object.Equals(A,null)) return 0;
                return A.Size.NumberOfElements;   
            }
        }
    }
}
