///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics.Storage;
using ILNumerics.Misc;
using ILNumerics.Exceptions; 

namespace ILNumerics  {
    public partial class ILMath {

        /// <summary>
        /// Minimum of all elements of array A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns><para>Scalar minimum of all elements of A.</para>
        /// <para>If A is empty, an empty array will be returned.</para></returns>
        /// <exception cref="ILNumerics.Exceptions.ILArgumentException">If A was null.</exception>
        /// <seealso cref="ILNumerics.ILMath.min(ILInArray{double},ILOutArray{double},int)"/>
        public static ILRetArray<double> minall(ILInArray<double> A ) {
            if (object.Equals(A, null))
                throw new ILArgumentException("input argument must not be null");
            using (ILScope.Enter(A)) {
                if (A.IsEmpty) {
                    return empty<double>(ILSize.Empty00);
                }
                
                double retVal = double.MaxValue;
                unsafe {
                    fixed (double* inArrStart = A.GetArrayForRead()) {
                          double* inArrWalk = inArrStart;
                         double* inArrEnd = inArrStart + A.Size.NumberOfElements;
                        while (inArrWalk < inArrEnd) {
                            if (retVal > *inArrWalk)
                                retVal = *inArrWalk;
                            inArrWalk++;
                        }
                    }
                }
                return array<double>(retVal , 1, 1);
            }
        }

#region HYCALPER AUTO GENERATED CODE

        /// <summary>
        /// Minimum of all elements of array A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns><para>Scalar minimum of all elements of A.</para>
        /// <para>If A is empty, an empty array will be returned.</para></returns>
        /// <exception cref="ILNumerics.Exceptions.ILArgumentException">If A was null.</exception>
        /// <seealso cref="ILNumerics.ILMath.min(ILInArray{double},ILOutArray{double},int)"/>
        public static ILRetArray<Int64> minall(ILInArray<Int64> A ) {
            if (object.Equals(A, null))
                throw new ILArgumentException("input argument must not be null");
            using (ILScope.Enter(A)) {
                if (A.IsEmpty) {
                    return empty<Int64>(ILSize.Empty00);
                }
               
                Int64 retVal = Int64.MaxValue;
                unsafe {
                    fixed (Int64* inArrStart = A.GetArrayForRead()) {
                         Int64* inArrWalk = inArrStart;
                        Int64* inArrEnd = inArrStart + A.Size.NumberOfElements;
                        while (inArrWalk < inArrEnd) {
                            if (retVal > *inArrWalk)
                                retVal = *inArrWalk;
                            inArrWalk++;
                        }
                    }
                }
                return array<Int64>(retVal , 1, 1);
            }
        }
        /// <summary>
        /// Minimum of all elements of array A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns><para>Scalar minimum of all elements of A.</para>
        /// <para>If A is empty, an empty array will be returned.</para></returns>
        /// <exception cref="ILNumerics.Exceptions.ILArgumentException">If A was null.</exception>
        /// <seealso cref="ILNumerics.ILMath.min(ILInArray{double},ILOutArray{double},int)"/>
        public static ILRetArray<Int32> minall(ILInArray<Int32> A ) {
            if (object.Equals(A, null))
                throw new ILArgumentException("input argument must not be null");
            using (ILScope.Enter(A)) {
                if (A.IsEmpty) {
                    return empty<Int32>(ILSize.Empty00);
                }
               
                Int32 retVal = Int32.MaxValue;
                unsafe {
                    fixed (Int32* inArrStart = A.GetArrayForRead()) {
                         Int32* inArrWalk = inArrStart;
                        Int32* inArrEnd = inArrStart + A.Size.NumberOfElements;
                        while (inArrWalk < inArrEnd) {
                            if (retVal > *inArrWalk)
                                retVal = *inArrWalk;
                            inArrWalk++;
                        }
                    }
                }
                return array<Int32>(retVal , 1, 1);
            }
        }
        /// <summary>
        /// Minimum of all elements of array A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns><para>Scalar minimum of all elements of A.</para>
        /// <para>If A is empty, an empty array will be returned.</para></returns>
        /// <exception cref="ILNumerics.Exceptions.ILArgumentException">If A was null.</exception>
        /// <seealso cref="ILNumerics.ILMath.min(ILInArray{double},ILOutArray{double},int)"/>
        public static ILRetArray<byte> minall(ILInArray<byte> A ) {
            if (object.Equals(A, null))
                throw new ILArgumentException("input argument must not be null");
            using (ILScope.Enter(A)) {
                if (A.IsEmpty) {
                    return empty<byte>(ILSize.Empty00);
                }
               
                byte retVal = byte.MaxValue;
                unsafe {
                    fixed (byte* inArrStart = A.GetArrayForRead()) {
                         byte* inArrWalk = inArrStart;
                        byte* inArrEnd = inArrStart + A.Size.NumberOfElements;
                        while (inArrWalk < inArrEnd) {
                            if (retVal > *inArrWalk)
                                retVal = *inArrWalk;
                            inArrWalk++;
                        }
                    }
                }
                return array<byte>(retVal , 1, 1);
            }
        }
        /// <summary>
        /// Minimum of all elements of array A
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns><para>Scalar minimum of all elements of A.</para>
        /// <para>If A is empty, an empty array will be returned.</para></returns>
        /// <exception cref="ILNumerics.Exceptions.ILArgumentException">If A was null.</exception>
        /// <seealso cref="ILNumerics.ILMath.min(ILInArray{double},ILOutArray{double},int)"/>
        public static ILRetArray<float> minall(ILInArray<float> A ) {
            if (object.Equals(A, null))
                throw new ILArgumentException("input argument must not be null");
            using (ILScope.Enter(A)) {
                if (A.IsEmpty) {
                    return empty<float>(ILSize.Empty00);
                }
               
                float retVal = float.MaxValue;
                unsafe {
                    fixed (float* inArrStart = A.GetArrayForRead()) {
                         float* inArrWalk = inArrStart;
                        float* inArrEnd = inArrStart + A.Size.NumberOfElements;
                        while (inArrWalk < inArrEnd) {
                            if (retVal > *inArrWalk)
                                retVal = *inArrWalk;
                            inArrWalk++;
                        }
                    }
                }
                return array<float>(retVal , 1, 1);
            }
        }

#endregion HYCALPER AUTO GENERATED CODE

    }
}
