///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics.Storage;
using ILNumerics.Misc;
using ILNumerics.Exceptions;



namespace ILNumerics  {
    
    public partial class ILMath {
    

        /// <summary>
        /// Convert real array to fcomplex array
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>fcomplex array with A as real part, imaginary part is zero</returns>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< double > A) {
            using (ILScope.Enter(A)) {
                int nrX = A.Size.NumberOfElements;
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, A.Size);
                
                double[] inArr = A.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = new fcomplex((float)inArr[i], 0f);
                }
                return ret;
            }
        }
        /// <summary>
        /// Create fcomplex array out of real and imaginary parts
        /// </summary>
        /// <param name="real">Real array for real part</param>
        /// <param name="imag">Real array for imaginary part</param>
        /// <returns>fcomplex array having the real- and part imaginary parts constructed out of 
        /// real and imag.</returns>
        /// <remarks>real and imag must have the same number of elements.
        /// The array returned will have the same size as the input arrays.</remarks>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< double > real, ILInArray< double > imag) {
            using (ILScope.Enter(real, imag)) {
                int nrX = real.Size.NumberOfElements;
                if (nrX != imag.Size.NumberOfElements)
                    throw new ILArgumentSizeException("size of real and imag input arrays must match!");
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, real.Size);
                
                double[] inArrReal = real.GetArrayForRead();
                
                double[] inArrImag = imag.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i].real = (float)inArrReal[i];
                    retArr[i].imag = (float)inArrImag[i];
                }
                return ret;
            }
        }


#region HYCALPER AUTO GENERATED CODE

        /// <summary>
        /// Convert real array to fcomplex array
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>fcomplex array with A as real part, imaginary part is zero</returns>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< byte > A) {
            using (ILScope.Enter(A)) {
                int nrX = A.Size.NumberOfElements;
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, A.Size);
               
                byte[] inArr = A.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = new fcomplex((float)inArr[i], 0f);
                }
                return ret;
            }
        }
        /// <summary>
        /// Create fcomplex array out of real and imaginary parts
        /// </summary>
        /// <param name="real">Real array for real part</param>
        /// <param name="imag">Real array for imaginary part</param>
        /// <returns>fcomplex array having the real- and part imaginary parts constructed out of 
        /// real and imag.</returns>
        /// <remarks>real and imag must have the same number of elements.
        /// The array returned will have the same size as the input arrays.</remarks>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< byte > real, ILInArray< byte > imag) {
            using (ILScope.Enter(real, imag)) {
                int nrX = real.Size.NumberOfElements;
                if (nrX != imag.Size.NumberOfElements)
                    throw new ILArgumentSizeException("size of real and imag input arrays must match!");
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, real.Size);
               
                byte[] inArrReal = real.GetArrayForRead();
               
                byte[] inArrImag = imag.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i].real = (float)inArrReal[i];
                    retArr[i].imag = (float)inArrImag[i];
                }
                return ret;
            }
        }

        /// <summary>
        /// Convert real array to fcomplex array
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>fcomplex array with A as real part, imaginary part is zero</returns>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< Int64 > A) {
            using (ILScope.Enter(A)) {
                int nrX = A.Size.NumberOfElements;
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, A.Size);
               
                Int64[] inArr = A.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = new fcomplex((float)inArr[i], 0f);
                }
                return ret;
            }
        }
        /// <summary>
        /// Create fcomplex array out of real and imaginary parts
        /// </summary>
        /// <param name="real">Real array for real part</param>
        /// <param name="imag">Real array for imaginary part</param>
        /// <returns>fcomplex array having the real- and part imaginary parts constructed out of 
        /// real and imag.</returns>
        /// <remarks>real and imag must have the same number of elements.
        /// The array returned will have the same size as the input arrays.</remarks>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< Int64 > real, ILInArray< Int64 > imag) {
            using (ILScope.Enter(real, imag)) {
                int nrX = real.Size.NumberOfElements;
                if (nrX != imag.Size.NumberOfElements)
                    throw new ILArgumentSizeException("size of real and imag input arrays must match!");
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, real.Size);
               
                Int64[] inArrReal = real.GetArrayForRead();
               
                Int64[] inArrImag = imag.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i].real = (float)inArrReal[i];
                    retArr[i].imag = (float)inArrImag[i];
                }
                return ret;
            }
        }

        /// <summary>
        /// Convert real array to fcomplex array
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>fcomplex array with A as real part, imaginary part is zero</returns>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< Int32 > A) {
            using (ILScope.Enter(A)) {
                int nrX = A.Size.NumberOfElements;
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, A.Size);
               
                Int32[] inArr = A.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = new fcomplex((float)inArr[i], 0f);
                }
                return ret;
            }
        }
        /// <summary>
        /// Create fcomplex array out of real and imaginary parts
        /// </summary>
        /// <param name="real">Real array for real part</param>
        /// <param name="imag">Real array for imaginary part</param>
        /// <returns>fcomplex array having the real- and part imaginary parts constructed out of 
        /// real and imag.</returns>
        /// <remarks>real and imag must have the same number of elements.
        /// The array returned will have the same size as the input arrays.</remarks>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< Int32 > real, ILInArray< Int32 > imag) {
            using (ILScope.Enter(real, imag)) {
                int nrX = real.Size.NumberOfElements;
                if (nrX != imag.Size.NumberOfElements)
                    throw new ILArgumentSizeException("size of real and imag input arrays must match!");
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, real.Size);
               
                Int32[] inArrReal = real.GetArrayForRead();
               
                Int32[] inArrImag = imag.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i].real = (float)inArrReal[i];
                    retArr[i].imag = (float)inArrImag[i];
                }
                return ret;
            }
        }

        /// <summary>
        /// Convert real array to fcomplex array
        /// </summary>
        /// <param name="A">Input array</param>
        /// <returns>fcomplex array with A as real part, imaginary part is zero</returns>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< float > A) {
            using (ILScope.Enter(A)) {
                int nrX = A.Size.NumberOfElements;
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, A.Size);
               
                float[] inArr = A.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i] = new fcomplex((float)inArr[i], 0f);
                }
                return ret;
            }
        }
        /// <summary>
        /// Create fcomplex array out of real and imaginary parts
        /// </summary>
        /// <param name="real">Real array for real part</param>
        /// <param name="imag">Real array for imaginary part</param>
        /// <returns>fcomplex array having the real- and part imaginary parts constructed out of 
        /// real and imag.</returns>
        /// <remarks>real and imag must have the same number of elements.
        /// The array returned will have the same size as the input arrays.</remarks>
        public static ILRetArray<fcomplex> real2fcomplex (ILInArray< float > real, ILInArray< float > imag) {
            using (ILScope.Enter(real, imag)) {
                int nrX = real.Size.NumberOfElements;
                if (nrX != imag.Size.NumberOfElements)
                    throw new ILArgumentSizeException("size of real and imag input arrays must match!");
                fcomplex[] retArr = ILMemoryPool.Pool.New<fcomplex>(nrX);
                ILRetArray<fcomplex> ret = new ILRetArray<fcomplex>(retArr, real.Size);
               
                float[] inArrReal = real.GetArrayForRead();
               
                float[] inArrImag = imag.GetArrayForRead();
                for (int i = 0; i < nrX; i++) {
                    retArr[i].real = (float)inArrReal[i];
                    retArr[i].imag = (float)inArrImag[i];
                }
                return ret;
            }
        }


#endregion HYCALPER AUTO GENERATED CODE

    }

}
